import os
import pandas as pd
import timeit
import traceback

from pathlib import Path
from string import Template
from time import sleep

from seaschedule import maeu
from seaschedule.config import CFG, RUN_TIME
from seaschedule.utils.email import send_mail
from seaschedule.utils.log import get_logger, format_log_message
from seaschedule.utils.sftp import put_files


RUN_TIME_STR = RUN_TIME.strftime("%Y%m%d_%H%M%S")
STATUS_PENDING = "PENDING"
STATUS_DISABLED = "DISABLED"
STATUS_SKIPPED = "SKIPPED"
STATUS_FAILED = "FAILED"
STATUS_SUCCESSFUL = "SUCCESSFUL"


def get_log_dir() -> Path:
    # Use path in config.toml if available, otherwise use <home-path>\seaschedule\log
    if CFG["environment"]["directory"]["log"]:
        log_dir = Path(CFG["environment"]["directory"]["log"])
    else:
        log_dir = Path.home().joinpath("seaschedule", "log")
    return log_dir


def get_data_dir() -> Path:
    # Use path in config.toml if available, otherwise use <home-path>\seaschedule\data
    if CFG["environment"]["directory"]["data"]:
        data_dir = Path(CFG["environment"]["directory"]["data"])
    else:
        data_dir = Path.home().joinpath("seaschedule", "data")
    return data_dir


def get_api_key(carrier: str) -> str:
    api_key = CFG[carrier]["api_key"]
    if not api_key:
        api_key = os.environ[CFG["environment"][carrier]["api_key"]]
    return api_key


def get_sftp_credential(app: str) -> tuple:
    sftp_credential = (os.environ[CFG["environment"][app]["sftp"]["username"]],
                        os.environ[CFG["environment"][app]["sftp"]["password"]])
    return sftp_credential


def get_smtp_host() -> str:
    smtp_host = os.environ[CFG["environment"]["smtp"]["host"]]
    return smtp_host


def main() -> None:
    # Init
    # Logging
    execution_start = timeit.default_timer()
    execution_elapsed = timeit.default_timer() - execution_start
    execution_successful = False
    task_step = 0
    task_name = ""
    task_start = timeit.default_timer()
    task_elapsed = timeit.default_timer() - task_start
    task_logs = []
    # Get directories and create them if not exists
    log_dir = get_log_dir()
    log_dir.mkdir(parents=True, exist_ok=True)
    log_file = log_dir.joinpath(CFG["logging"]["filename"].format(RUN_TIME_STR))
    data_dir = get_data_dir()
    data_dir.mkdir(parents=True, exist_ok=True)
    # Initialize logger
    logger = get_logger(name=CFG["logging"]["logger"], log_file=str(log_file),
                        log_level=CFG["logging"]["level"], log_format=CFG["logging"]["format"])
    logger.info("Process started!")

    try:
        api_key = get_api_key("maeu")

        # Step-1: Retrieving active vessels
        task_step += 1
        task_name = "Retrieving active vessels"
        task_start = timeit.default_timer()
        vessels_file = data_dir.joinpath(CFG["maeu"]["vessels"]["filename"].format(RUN_TIME_STR, ".csv"))
        vessels_source_file = data_dir.joinpath(CFG["maeu"]["vessels"]["filename"].format(RUN_TIME_STR, ".txt"))
        vessel_master = pd.DataFrame()
        vessel_source_master = ""
        imos = []
        max_try = CFG["maeu"]["vessels"]["max_try"]
        retry_second = CFG["maeu"]["vessels"]["retry_second"]

        for i in range(1, max_try + 1):
            res_code = 0
            try:
                vessel_master, res_code, vessel_source_master = maeu.get_active_vessels(api_key)
                if not vessel_master.empty:
                    imos = vessel_master["imo"].tolist()
                    vessel_master.to_csv(vessels_file, encoding="utf-8", index=False)
                    with open(str(vessels_source_file), "w", encoding="utf-8") as f:
                        f.write(vessel_source_master)
                    task_elapsed = timeit.default_timer() - task_start
                    logger.info(
                        f"Step-{task_step}: {task_name} completed successfully in {task_elapsed:.2f} seconds"
                        f", # of active vessels = {len(imos)}.")
                    task_logs.append({"step": str(task_step), "name": task_name, "status": STATUS_SUCCESSFUL,
                                      "remarks": f"# of active vessels = {len(imos)}", "seconds": task_elapsed})
                    break
                else:
                    raise ValueError(vessel_source_master)
            except Exception as e:
                if i >= max_try:
                    # Stop retry and rethrow exception if this is the last try
                    raise
                else:
                    logger.warning(
                        f"Step-{task_step}: {task_name} try #{i} failed: {e}. Retry in {retry_second} seconds.")
                    sleep(retry_second)

        # Step-2: Retrieving vessel schedules
        task_step += 1
        task_name = "Retrieving vessel schedules"
        task_start = timeit.default_timer()
        schedules_file = data_dir.joinpath(CFG["maeu"]["schedules"]["filename"].format(RUN_TIME_STR, ".csv"))
        schedules_source_file = data_dir.joinpath(CFG["maeu"]["schedules"]["filename"].format(RUN_TIME_STR, ".json"))
        schedule_master = pd.DataFrame()
        schedule_source_master = ""
        carriers = CFG["maeu"]["schedules"]["carriers"]
        max_imo = CFG["maeu"]["schedules"]["max_imo"]
        imo_count = 0
        request_count = 0
        schedule_count = 0
        max_try = CFG["maeu"]["schedules"]["max_try"]
        retry_second = CFG["maeu"]["schedules"]["retry_second"]

        # DEBUG
        # max_imo = 20
        # imos = ["9857157"]
        # carriers = ["MAEU"]

        for imo in imos:
            if max_imo != -1 and imo_count >= max_imo:
                break
            for carrier in carriers:
                for i in range(1, max_try + 1):
                    schedule_data = pd.DataFrame()
                    res_code = 0
                    res_text = ""
                    try:
                        request_count += 1
                        logger.info(f"> [{request_count}] Retrieving schedule of {imo}/{carrier}.")
                        # Get schedule by imo + carrier
                        schedule_data, res_code, res_text = \
                            maeu.get_vessel_schedule(api_key, imo, carrier, RUN_TIME.strftime("%Y-%m-%d"),
                                                     CFG["maeu"]["schedules"]["data_range"])
                        if not schedule_data.empty:
                            if not schedule_source_master:
                                schedule_source_master = res_text
                            else:
                                schedule_source_master = schedule_source_master + ",\n" + res_text
                            # Append schedule to master
                            schedule_master = pd.concat([schedule_master, schedule_data], ignore_index=True)
                            logger.info(f"> [{request_count}] Retrieving schedule of {imo}/{carrier} completed.")
                            schedule_count += 1
                            break
                        else:
                            raise ValueError(res_text)
                    except Exception as e:
                        if i >= max_try or "Data not found" in str(e):
                            # Stop retry if this is the last try or no data for this imo
                            # error_msg = format_log_message(traceback.format_exc() + " --- " + str(e))
                            logger.error(
                                f"> [{request_count}] Retrieving schedule of {imo}/{carrier} failed.")
                            break
                        else:
                            logger.warning(
                                f"> [{request_count}] Retrieving schedule of {imo}/{carrier} failed. Retry in {retry_second} seconds.")
                            sleep(retry_second)
            imo_count += 1
        if not schedule_master.empty:
            schedule_master.to_csv(schedules_file, encoding="utf-8", index=False)
            with open(str(schedules_source_file), "w", encoding="utf-8") as f:
                # Add square brackets to make the consolidated responses a valid json document
                f.write("[" + schedule_source_master + "]")
            task_elapsed = timeit.default_timer() - task_start
            logger.info(
                f"Step-{task_step}: {task_name} completed successfully in {task_elapsed:.2f} seconds"
                f", # of schedules retrieved = {schedule_count}.")
            task_logs.append({"step": str(task_step), "name": task_name, "status": STATUS_SUCCESSFUL,
                              "remarks": f"# of schedules retrieved = {schedule_count}", "seconds": task_elapsed})
        else:
            raise ValueError("No vessel schedule can be retrieved.")

        # Task-3: Uploading to SFTP
        task_step += 1
        task_name = "Uploading to SFTP"
        task_start = timeit.default_timer()
        if not CFG["ib"]["sftp"]["enable"]:
            task_logs.append({"step": str(task_step), "name": task_name, "status": STATUS_DISABLED, "remarks": "",
                              "seconds": float("nan")})
        elif not schedules_file.exists():
            raise ValueError("No schedule file can be found for uploading.")
        else:
            max_try = 5
            retry_second = 5

            for i in range(1, max_try + 1):
                try:
                    sftp_username, sftp_password = get_sftp_credential("ib")
                    put_files(CFG["ib"]["sftp"]["host"], CFG["ib"]["sftp"]["port"], sftp_username, sftp_password,
                              [schedules_file], CFG["ib"]["sftp"]["remote_directory"])
                    task_elapsed = timeit.default_timer() - task_start
                    logger.info(
                        f"Step-{task_step}: {task_name} completed successfully in {task_elapsed:.2f} seconds.")
                    task_logs.append({"step": str(task_step), "name": task_name, "status": STATUS_SUCCESSFUL,
                                      "remarks": f"uploaded file = {Path(schedules_file).name}", "seconds": task_elapsed})
                    break
                except Exception as e:
                    if i >= max_try:
                        # Stop retry and rethrow exception if this is the last try
                        raise
                    else:
                        logger.warning(f"Step-{task_step}: {task_name} try #{i} failed: {e}. Retry in {retry_second} seconds.")
                        sleep(retry_second)
        execution_successful = True
    except Exception as e:  # AssertionError
        # logger.error(format_log_message(traceback.format_exc() + " --- " + str(e)))
        error_msg = format_log_message(traceback.format_exc() + " --- " + str(e))
        task_elapsed = timeit.default_timer() - task_start
        logger.error(f"Step-{task_step}: {task_name} failed: {error_msg}.")
        task_logs.append({"step": str(task_step), "name": task_name, "status": STATUS_FAILED,
                          "remarks": "", "seconds": task_elapsed})
    finally:
        # End
        execution_elapsed = timeit.default_timer() - execution_start
        logger.info(f"Process completed in {execution_elapsed:.2f} seconds.")

    # Sending notification email
    try:
        host = get_smtp_host()
        from_addr = CFG["email"]["from"]
        to_addrs = CFG["email"]["to"]
        cc_addrs = CFG["email"]["cc"]
        subject = CFG["email"]["subject"]
        body_template = Template(CFG["email"]["body"])
        if execution_successful:
            result_line = f'The process is completed <span style="font-weight:bold; color:green">' \
                          f'SUCCESSFULLY</span> in {execution_elapsed:.2f} seconds.'
        else:
            result_line = f'The process is <span style="font-weight:bold; color:red">' \
                          f'FAILED</span>. Please check the attached log for details.'
        log_summary = ""
        for task_log in task_logs:
            log_summary = log_summary + f'- Step-{task_log["step"]}: {task_log["name"]}' \
                                        f'......... {task_log["status"]} ({task_log["remarks"]})' \
                                        f' ({task_log["seconds"]:.2f}s)<br>'
        body = body_template.substitute({"result_line": result_line, "log_summary": log_summary})
        attachments = []
        if log_file.exists():
            attachments = [str(log_file)]
        send_mail(host=host, port=25, from_addr=from_addr, to_addrs=to_addrs, cc_addrs=cc_addrs,
                  subject=subject, body=body, files=attachments)
    except Exception as e:
        error_msg = format_log_message(traceback.format_exc() + " --- " + str(e))
        logger.error(f"Sending notification email failed: {error_msg}.")


if __name__ == "__main__":
    main()



import smtplib

from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.application import MIMEApplication
from pathlib import Path


def send_mail(host: str, port: int, from_addr: str, to_addrs: str, cc_addrs: str,
              subject: str, body: str, files: list) -> None:
    msg = MIMEMultipart()
    msg["from"] = from_addr
    msg["to"] = to_addrs
    if cc_addrs: msg["cc"] = cc_addrs
    msg["subject"] = subject
    msg.attach(MIMEText(body, "html"))
    for file in files:
        with open(file, "rb") as f:
            attachment = MIMEApplication(f.read())
            attachment.add_header("Content-Disposition", "attachment", filename=Path(file).name)
            msg.attach(attachment)

    with smtplib.SMTP(host=host, port=port) as smtp:
        smtp.send_message(msg)



import logging


def get_logger(name: str, log_file: str, log_level: int, log_format: str) -> logging.Logger:
    logger = logging.getLogger(name)
    logger.setLevel(log_level)
    formatter = logging.Formatter(log_format)

    # Console handler
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)
    # File handler
    file_handler = logging.FileHandler(log_file)
    file_handler.setFormatter(formatter)
    # Add handlers
    logger.addHandler(console_handler)
    logger.addHandler(file_handler)

    return logger


def format_log_message(message: str = "") -> str:
    return message.replace("\n", "\\n")



import os

from datetime import datetime
from importlib import resources
try:
    import tomllib
except ModuleNotFoundError:
    import tomli as tomllib


# Read environment
try:
    ENV = os.environ["PYTHON_ENV"]
    CFG_FILE = f"config_{ENV}.toml"
except KeyError as e:
    ENV = ""
    CFG_FILE = f"config.toml"

# Read configuration file
CFG = tomllib.loads(resources.read_text("seaschedule.config", CFG_FILE, encoding="utf-8"))

# Additional configurations
RUN_TIME = datetime.now()



import paramiko

from pathlib import Path


def put_files(host: str, port: int, username: str, password: str, files: list, remote_directory: str) -> None:
    with paramiko.SSHClient() as ssh:
        # ssh.load_system_host_keys()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(host, port=port, username=username, password=password)
        sftp = ssh.open_sftp()
        for file in files:
            sftp.put(file, remote_directory + "/" + Path(file).name)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['seaschedule', 'seaschedule.config', 'seaschedule.utils']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.5.1,<2.0.0',
 'paramiko>=2.11.0,<3.0.0',
 'requests>=2.28.1,<3.0.0',
 'tomli>=2.0.1,<3.0.0']

entry_points = \
{'console_scripts': ['ss = seaschedule.__main__:main']}

setup_kwargs = {
    'name': 'seaschedule',
    'version': '0.0.2',
    'description': 'Get sea schedules from Maersk and other shipping lines.',
    'long_description': '# seaschedule\nGet sea schedules from Maersk and other shipping lines.\n\n## Installation\nFrom [PyPI](https://pypi.org/project/seaschedule/):\n\n    python -m pip install seaschedule\n\n## Setup\nThe following setup must be done before running seaschedule:\n1. Create below environment variables in your OS environment:\n    * `SS_SMTP_HOST`: SMTP host for sending notification emails\n    * `SS_IB_SFTP_USER`: SFTP user for uploading schedule files to Information Broker\n    * `SS_IB_SFTP_PWD`: SFTP password for uploading schedule files to Information Broker\n    * `SS_MAEU_API_KEY`: API key given by Maersk \n<br/><br/>\n2. Specify below directory paths in `site-packages\\seaschedule\\config\\config.toml` for storing the schedule data files and log files. For example:\n    ```\n    [environment]\n    directory.data = "/home/user1/seaschedule/data"\n    directory.log = "/home/user1/seaschedule/log"\n\n    # Windows\n    directory.data = "C:\\Users\\user1\\Documents\\seaschedule\\data"\n    directory.log = "C:\\Users\\user1\\Documents\\seaschedule\\data"\n    ```\n\n## How to Use\nseaschedule is a console application, named `seaschedule`.\n\n    >>> python -m seaschedule\n',
    'author': 'Alex Cheng',
    'author_email': 'alex28.biz@gmail.com',
    'maintainer': 'Alex Cheng',
    'maintainer_email': 'alex28.biz@gmail.com',
    'url': 'https://github.com/alexcheng628/seaschedule',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import json
import pandas as pd
import requests

from datetime import datetime
from operator import itemgetter

RUN_TIME = datetime.now()
RUN_TIME_STR = RUN_TIME.strftime("%Y-%m-%dT%H:%M:%S")
INTERNAL_FACILITY_TERMINAL_GEOID = ["2ON3JDAEYWYES", "3UIO1T0SG68FO", "1NEO0LDYASVG4"]


def get_active_vessels(api_key: str) -> tuple:
    assert api_key, "The maeu_api_key is empty."

    endpoint = "https://api.maersk.com/schedules/active-vessels"
    request_headers = {'Consumer-Key': api_key, 'accept': 'application/stream+json'}
    vessel_cols = ["imo", "name"]
    vessel_data = pd.DataFrame()

    # TODO: Add certificates, verify=False, verify="www-maersk-com-chain.pem"
    res = requests.get(endpoint, headers=request_headers, verify=False)
    res_code = res.status_code
    res_text = res.text
    if res.ok:
        vessels= res_text.split("\n")
        for index, line in enumerate(vessels):
            # Skip the empty lines and vessels without IMO
            if line and 'vesselIMONumber' in line:
                vessel = json.loads(line)
                if vessel:
                    row_df = pd.DataFrame([[vessel.get("vesselIMONumber", ""), vessel.get("vesselName", "")]],
                                          columns=vessel_cols)
                    vessel_data = pd.concat([vessel_data, row_df], ignore_index=True)
    return vessel_data, res_code, res_text


def get_vessel_schedule(api_key: str, imo: str, carrier: str, start_date: str, date_range: str = "P12W") -> tuple:
    assert api_key, "The maeu_api_key is empty."

    endpoint = "https://api.maersk.com/schedules/vessel-schedules"
    request_headers = {'Consumer-Key': api_key, 'accept': 'application/stream+json'}
    url_params = {"vesselIMONumber": imo, "startDate": start_date, "dateRange": date_range, "carrierCodes": carrier}
    schedule_cols = ["carrierCode", "vesselFlagCode", "vesselIMONumber", "vesselName", "vesselCallOrder",
                   "estimatedDepartureTime", "actualDepartureTime", "estimatedArrivalTime", "actualArrivalTime", "facilityCode",
                   "facilityCityGeoID", "facilityTerminalCode", "facilityTerminalGeoID", "facilityCityName", "facilityCountryCode",
                   "facilityCountryName", "facilityLocationName", "facilityPortName", "serviceCode", "serviceName",
                   "inboundVoyageNumber", "voyageNumber"]
    # schedule_data = pd.DataFrame([], columns=schedule_cols)
    schedule_data = pd.DataFrame()

    # TODO: Add certificates
    res = requests.get(endpoint, headers=request_headers, params=url_params, verify=False)
    res_code = res.status_code
    res_text = res.text
    if res.ok:
        stops = []
        schedule = json.loads(res_text)
        # carrier
        carrierCode = carrier
        # vessel
        vessel = schedule["vessel"]
        # vesselCode = schedule["vessel"].get("carrierVesselCode", "")
        # vesselCallSign = vessel.get("vesselCallSign", "")
        vesselFlagCode = vessel.get("vesselFlagCode", "")
        vesselIMONumber = vessel.get("vesselIMONumber", "")
        vesselName = vessel.get("vesselName", "")
        # vesselCalls
        vesselCalls = schedule["vesselCalls"]
        for vesselCall in vesselCalls:
            vesselCallOrder = "TBD"
            # callSchedules
            callSchedules = vesselCall["callSchedules"]
            estimatedDepartureTime = ""
            actualDepartureTime = ""
            estimatedArrivalTime = ""
            actualArrivalTime = ""
            for callSchedule in callSchedules:
                # Parse ETD / ATD / ETA / ATA based on 'eventClassifierCode' and 'transportEventTypeCode'
                if callSchedule.get("eventClassifierCode", "") == "EST" and callSchedule.get("transportEventTypeCode",
                                                                                             "") == "DEPA":
                    estimatedDepartureTime = callSchedule.get("classifierDateTime", "")
                elif callSchedule.get("eventClassifierCode", "") == "EST" and callSchedule.get("transportEventTypeCode",
                                                                                               "") == "ARRI":
                    estimatedArrivalTime = callSchedule.get("classifierDateTime", "")
                elif callSchedule.get("eventClassifierCode", "") == "ACT" and callSchedule.get("transportEventTypeCode",
                                                                                               "") == "DEPT":
                    actualDepartureTime = callSchedule.get("classifierDateTime", "")
                elif callSchedule.get("eventClassifierCode", "") == "ACT" and callSchedule.get("transportEventTypeCode",
                                                                                               "") == "ARRI":
                    actualArrivalTime = callSchedule.get("classifierDateTime", "")
            # According to https://developer.maersk.com/api-catalogue/Vessel%20Schedules, this version of the API
            # classifies all events as EST Estimated and makes no distinction whether or not the event has been actualised.
            # Generally, if an event is reported for a past date, it is assumed to be actualised.
            # Therefore assume ATD/ATA equals to ETD/ETA if ETD/ETA is reported a past date.
            if (not actualDepartureTime) and estimatedDepartureTime and estimatedDepartureTime < RUN_TIME_STR:
                actualDepartureTime = estimatedDepartureTime
            if (not actualArrivalTime) and estimatedArrivalTime and estimatedArrivalTime < RUN_TIME_STR:
                actualArrivalTime = estimatedArrivalTime
            # facility
            facilityCode = vesselCall["facility"].get("UNLocationCode", "")
            facilityCityGeoID = vesselCall["facility"].get("carrierCityGeoID", "")
            facilityTerminalCode = vesselCall["facility"].get("carrierTerminalCode", "")
            facilityTerminalGeoID = vesselCall["facility"].get("carrierTerminalGeoID", "")
            facilityCityName = vesselCall["facility"].get("cityName", "")
            facilityCountryCode = vesselCall["facility"].get("countryCode", "")
            facilityCountryName = vesselCall["facility"].get("countryName", "")
            facilityLocationName = vesselCall["facility"].get("locationName", "")
            # facilityLocationType = vesselCall["facility"].get("locationType", "")
            facilityPortName = vesselCall["facility"].get("portName", "")
            # transport
            inboundServiceCode = vesselCall["transport"]["inboundService"].get("carrierServiceCode", "")
            inboundServiceName = vesselCall["transport"]["inboundService"].get("carrierServiceName", "")
            inboundVoyageNumber = vesselCall["transport"]["inboundService"].get("carrierVoyageNumber", "")
            # outboundServiceCode = vesselCall["transport"]["outboundService"].get("carrierServiceCode", "")
            # outboundServiceName = vesselCall["transport"]["outboundService"].get("carrierServiceName", "")
            outboundVoyageNumber = vesselCall["transport"]["outboundService"].get("carrierVoyageNumber", "")
            # Ignore Maersk internal locations (e.g., "Off Hire Site", "On Hire Site", "Repair Yard Site")
            if not (facilityTerminalGeoID in INTERNAL_FACILITY_TERMINAL_GEOID):
                stop = [carrierCode, vesselFlagCode, vesselIMONumber, vesselName,
                        vesselCallOrder, estimatedDepartureTime, actualDepartureTime, estimatedArrivalTime,
                        actualArrivalTime, facilityCode, facilityCityGeoID, facilityTerminalCode,
                        facilityTerminalGeoID, facilityCityName, facilityCountryCode, facilityCountryName,
                        facilityLocationName, facilityPortName, inboundServiceCode, inboundServiceName,
                        inboundVoyageNumber, inboundVoyageNumber]
                stops.append(stop)
                if inboundVoyageNumber != outboundVoyageNumber:
                    stop_ob_voyage = [carrierCode, vesselFlagCode, vesselIMONumber, vesselName,
                                      vesselCallOrder, estimatedDepartureTime, actualDepartureTime, estimatedArrivalTime,
                                      actualArrivalTime, facilityCode, facilityCityGeoID, facilityTerminalCode,
                                      facilityTerminalGeoID, facilityCityName, facilityCountryCode, facilityCountryName,
                                      facilityLocationName, facilityPortName, inboundServiceCode, inboundServiceName,
                                      inboundVoyageNumber, outboundVoyageNumber]
                    stops.append(stop_ob_voyage)

        # Sort by voyageNumber and estimatedDepartureTime
        stops.sort(key=itemgetter(21, 5))

        # Assign vesselCallOrder after sorting
        vessel_call_order = 0
        this_voyage = ""
        prev_voyage = ""
        for index, stop in enumerate(stops):
            this_voyage = stop[21]
            if this_voyage != prev_voyage:
                # Restart order for a new voyage
                vessel_call_order = 0
                prev_voyage = this_voyage
            else:
                vessel_call_order += 1
            # Assign vesselCallOrder
            stop[4] = str(vessel_call_order)

        # row_df = pd.DataFrame([["val1", "val2", "val3"]], columns=schedule_cols)
        # schedule_data = pd.concat([schedule_data, row_df], ignore_index=True)
        schedule_data = pd.DataFrame(stops, columns=schedule_cols)
    return schedule_data, res_code, res_text
